## Expression for V(AB) in Pearce's model, as a function of fraction
## of common elements between A and B (c). This expression is derived
## in solvingModels.py.
pearceSum <- function(c){
    2/((2-c)*(c^2 + 1))
}

## Expression for V(AB) in Pearce's model, as a function of fraction
## of common elements between A and B (c) and fraction of context
## elements (x). The expression is derived in solvingModels.py.
pearceSumX <- function(c,x) {
    2*(1 + x)*(-2 - 3*x + 2*x^2 + 2*x^3 + c + 2*c*x)/((2 + x - c)*(-2 + x + c)*(1 + 2*x + 2*c*x + c^2))
}

## Figure: Left panel is the prediction of Pearce's model without
## context effects, right panel includes context. Note that it saves
## in a Figures/ directory, change the filename on the first line if
## you want otherwise.
pdf( "Figures/PearceSummation.pdf", height=3.5, width=8 )
par( mar=c(4,4,1,2), las=1, mfrow=c(1,2) )
plot(
    pearceSum,
    xlim=c(0,1),
    ylim=c(0,2),
    xaxs="i",
    yaxs="i",
    xlab="c (Fraction of common elements)",
    ylab=expression(V(AB)),
    yaxt="n",
    xaxt="n"
)
axis( 1, c(0,.25,.5,.75,1) )
axis( 2, c(0,.5,1,1.5,2) )
plot(
    function(x){pearceSumX(0,x)},
    xlim=c(0,1),
    ylim=c(0,2),
    xaxs="i",
    yaxs="i",
    xlab="x (Relative salience of context)",
    ylab=expression(V(AB)),
    yaxt="n",
    xaxt="n"
)
axis( 1, c(0,.25,.5,.75,1) )
axis( 2, c(0,.5,1,1.5,2) )
plot(function(x){pearceSumX(.5,x)}, from=0, to=1, add=TRUE, lty=2 )
plot(function(x){pearceSumX(.9,x)}, from=0, to=1, add=TRUE, lty=3 )
legend(
    "topright",
    legend=c("c=0","c=1/2","c=9/10"),
    lty=1:3,
    bty="n"
)
dev.off()

## This quantity enters both V(A) and V(B) at the end of a Kamin
## blocking design, according to Pearce's model.
blockingPearce <- function( c ) {
    (1-c) / (2-c)^2
}

## This quantity enters both V(A) and V(B) at the end of a Kamin
## blocking design, according to the replaced elements model (REM).
blockingREM <- function( r ) {
    .5*r*(1-r)
}

## Figure: Results of blocking experiments according to Pearce (left)
## and REM (right).
pdf( "Figures/BlockingPearceREM.pdf", height=3.5, width=8 )
par( las=1, mar=c(4,5,3,2), mfrow=c(1,2) )
### Pearce panel
plot(
    function(x){ 1 + blockingPearce(x) }, # V(A)
    xlim=c(0,1),
    ylim=c(0,1.5),
    xaxs="i",
    yaxs="i",
    xlab="c (Fraction of common elements)",
    ylab="",
    xaxt="n",
    yaxt="n",
    main="Pearce (1987)"
)
axis( 1, c(0,.25,.5,.75,1) )
##axis( 2, c(0,.5,1,1.5) )
axis(
    2,
    at=c(5/4,1,1/4),
    labels=c(
	expression( scriptstyle(frac(1,2)) * v[AB]^(2) + scriptstyle(frac(3,4))* v[A]^(1) ),
	expression( v[AB]^(2) ),
	expression( scriptstyle(frac(1,2))* v[AB]^(2) - scriptstyle(frac(1,4)) * v[A]^(1) )
    )
)
plot(
    function(x){ x^2 + blockingPearce(x) }, # V(B)
    from=0,
    to=1,
    add=TRUE,
    lty=2
)
legend(
    "bottomright",
    legend=c(
	expression(V[2](A)),
	expression(V[2](B))
    ),
    lty=c(1,2),
    bty="n"
)
abline( h=1, lty=3 )
### REM panel
plot(
    function(x){ 1 + blockingREM(x) }, # V(A)
    xlim=c(0,1),
    ylim=c(0,1.5),
    xaxs="i",
    yaxs="i",
    xlab="r (Fraction of replaced elements)",
    ylab="",
    xaxt="n",
    yaxt="n",
    main="Brandon et al. (2000)"
)
axis( 1, c(0,.25,.5,.75,1) )
#axis( 2, c(0,.5,1,1.5) )
axis( 2,
     at=c(0,1),
     labels=c(
	 expression( scriptstyle(frac(1,2)) * ( v[AB]^(2) - v[A]^(1) ) ),
	 expression( scriptstyle(frac(1,2)) * ( v[AB]^(2) + v[A]^(1) ) )
     )
     )
plot(
    blockingREM, # V(B)
    from=0,
    to=1,
    add=TRUE,
    lty=2
)
legend(
    .55, .875,
    legend=c(
	expression(V[2](A)),
	expression(V[2](B))
	),
    lty=c(1,2),
    bty="n"
)
text(
    0.5,
    1+blockingREM(0.5),
    expression( scriptstyle(frac(1,4))*v[AB]^(2) + scriptstyle(frac(7,8))*v[A]^(1) ),
    pos=3
)
points( c(.5,.5), blockingREM(.5)+c(0,1), pch=16 )
text(
    0.5,
    blockingREM(0.5),
    expression( scriptstyle(frac(1,4))*v[AB]^(2) - scriptstyle(frac(1,8))*v[A]^(1) ),
    pos=3
)
abline( h=1, lty=3 )
dev.off()
