# This is a companion file to the paper:
#
# Studying associative learning without solving learning equations,
# by Stefano Ghirlanda
#
# (in press at Journal of Mathematical Psychology as of June
# 2018). This file reproduces the calculations in the paper using the
# sympy symbolic computation package in python.

from sympy import *
init_printing( order='grevlex' )

# Define the variables we will use below. We have model parameters x,
# r, and c, and generalization coefficients:
var('x r c' )
var('gX_X gX_AB gA_X gB_X gA_A gA_B gB_B gA_AB gB_AB gAB_AB')

# These are the generalization coefficents for the Brandon et
# al. (2000) "replaced elements" model, with replacement fraction of
# r and relative context salience of x:
Grem = {
    gX_X:   x,
    gA_A:   1+x,
    gB_B:   1+x,
    gAB_AB: 2+x,
    gX_AB:  x*(1-r)**2,
    gA_X:   x*(1-r),
    gB_X:   x*(1-r),
    gA_B:   x*(1-r),
    gA_AB:  (1+x)*(1-r),
    gB_AB:  (1+x)*(1-r)
}

### This is the special case in which the context has zero salience:
Grem0 = { k:v.subs(x,0) for (k,v) in Grem.items() }

## These are the generalization coefficients for Pearce's (1987)
## "configural" model, with fraction of shared elements of c and
## fraction of context elements of x:
Gpearce = {
    gX_X:   S.One,
    gA_A:   S.One,
    gB_B:   S.One,
    gAB_AB: S.One,
    gX_AB:  x/(2-c-x),
    gA_X:   x/(1+x),
    gB_X:   x/(1+x),
    gA_B:   (c+x)**2 / (1+x)**2,
    gA_AB:  (1+x) / (2-c+x),
    gB_AB:  (1+x) / (2-c+x)
}

## This is the special case in which the context has zero salience:
Gpearce0 = { k:v.subs(x,0) for (k,v) in Gpearce.items() }

## This is the Rescorla & Wagner (1972) model with common elements and
## shared context:
Grw = {
    gX_X:   x,
    gA_A:   1+x,
    gB_B:   1+x,
    gAB_AB: 2-c+x,
    gX_AB:  x,
    gA_X:   x,
    gB_X:   x,
    gA_B:   c+x,
    gA_AB:  1+x,
    gB_AB:  1+x
}

# This is the special case in which the context has zero salience:
Grw0 = { k:v.subs(x,0) for (k,v) in Grw.items() }

# This function takes a dictionary and, in each element, replaces
# with 0 all occurrences of generalization coefficients that include
# the context:
def noContext( x ):
    y = x.copy()
    for g in [gA_X, gB_X, gX_X, gX_AB]:
        y = { k:v.limit( g, 0 ) for k,v in y.items() }
    return y

#
#
# Analysis of summation. The solution to the endpoint equations is
# calculated first in the general case with context, and then also
# without context.
#
#

# Additional variables:
var( 'cA cB cX VAB vA vB' )

# Matrix of coefficients for endpoint equations:
gSum = Matrix([
    [gA_A, gA_B, gA_X],
    [gA_B, gB_B, gB_X],
    [gA_X, gB_X, gX_X]
])

# Vector of unknowns:
cSum = Matrix([
    [cA],
    [cB],
    [cX]
])

# L.h.s of the endpoint equations:
epSum = gSum * cSum

# Solution of the endpoint equations:
cSolSum = solve(
    (
        epSum[0] - vA,
        epSum[1] - vB,
        epSum[2] - 0,
    ),
    (cA, cB, cX)
)

# Solution without context:
cSolSum0 = noContext( cSolSum )

# Associative strength of AB at test, with and without context:
VAB = ( cA*gA_AB + cB*gB_AB + cX*gX_AB ).subs( cSolSum )
VAB0 = ( cA*gA_AB + cB*gB_AB + cX*gX_AB ).subs( cSolSum0 )

# Replaced elements model:
VABrem = VAB.subs( Grem )
VABrem0 = VABrem.limit(x,0)

# Rescorla-Wagner model:
VABrw = VAB.subs( Grw )
VABrw0 = VABrw.limit(x,0)

# Pearce's model:
VABpearce = VAB.subs( Gpearce ).factor()
VABpearce0 = VABpearce.subs(x,0)

# Print a few results for convenience. Printing with context is
# disabled by default as the expressions are messy. Uncomment to
# print.
print( "V(AB) in summation, Pearce's model, no context:" )
print( VABpearce0 )
# print( "V(AB) in summation, Pearce's model, with context:" )
# print( VABpearce )
print( "V(AB) in summation, Rescorla & Wagner's model, no context:" )
print( VABrw0 )
# print( "V(AB) in summation, Rescorla & Wagner's model, with context:" )
# print( VABrw )
print( "V(AB) in summation, replaced elements model, no context:" )
print( VABrem0 )
# print( "V(AB) in summation, replaced elements model, with context:" )
# print( VABrem )


#
#
# Analysis of blocking
#
#

# Additional variables:
var('c1A c2AB v1A v2AB')

# There is only one endpoint equation per phase, which is easy to
# solve manually without the solve() function.

# Coefficient for phase 1:
c1A = v1A / gA_A

# At the end of phase 1, AB has this associative strength:
V1AB = v1A * gA_AB / gA_A

# Hence the coefficient established during phase 2 is:
c2AB = (v2AB - V1AB) / gAB_AB

# We can now derive general expressions for V2A and V2B:
V2A = c1A*gA_A + c2AB*gA_AB
V2B = c1A*gA_B + c2AB*gB_AB

# And then specialize them for the three models:

V2Apearce0 = V2A.subs( Gpearce0 )
V2Bpearce0 = V2B.subs( Gpearce0 )

V2Arem0 = V2A.subs( Grem0 )
V2Brem0 = V2B.subs( Grem0 )

V2Arw0 = V2A.subs( Grw0 )
V2Brw0 = V2B.subs( Grw0 )

# Print a few results for convenience. 
print( "V(A) and V(B) in blocking, Pearce's model:" )
print( V2Apearce0 )
print( V2Bpearce0 )
print( "V(A) and V(B) in blocking, Rescorla & Wagner's model:" )
print( V2Arw0 )
print( V2Brw0 )
print( "V(A) and V(B) in blocking, replaced elements model:" )
print( V2Arem0 )
print( V2Brem0 )
